@@ -17,8 +17,7 @@
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
-#ifndef MRN_MYSQL_COMPAT_H_
-#define MRN_MYSQL_COMPAT_H_
+#pragma once
 
 #include "mrn_mysql.h"
 
@@ -32,6 +31,11 @@
 #  define MRN_HAVE_MYSQL_TYPE_JSON
 #endif
 
+#if MYSQL_VERSION_ID >= 100302 && defined(MRN_MARIADB_P)
+#  define MRN_HAVE_MYSQL_TYPE_VARCHAR_COMPRESSED
+#  define MRN_HAVE_MYSQL_TYPE_BLOB_COMPRESSED
+#endif
+
 #if MYSQL_VERSION_ID < 50603
   typedef MYSQL_ERROR Sql_condition;
 #endif
@@ -51,15 +55,115 @@
 #endif
 
 #if MYSQL_VERSION_ID >= 50609
-#  define MRN_KEY_HAS_USER_DEFINED_KEYPARTS
+#  define MRN_KEY_HAVE_USER_DEFINED_KEYPARTS
 #endif
 
-#ifdef MRN_KEY_HAS_USER_DEFINED_KEYPARTS
+#ifdef MRN_KEY_HAVE_USER_DEFINED_KEYPARTS
 #  define KEY_N_KEY_PARTS(key) (key)->user_defined_key_parts
 #else
 #  define KEY_N_KEY_PARTS(key) (key)->key_parts
 #endif
 
+#if (MYSQL_VERSION_ID >= 50706 && !defined(MRN_MARIADB_P)) || \
+    (MYSQL_VERSION_ID >= 100302)
+#  define MRN_FOREIGN_KEY_USE_CONST_STRING
+#endif
+
+#ifdef MRN_FOREIGN_KEY_USE_CONST_STRING
+  typedef LEX_CSTRING mrn_foreign_key_name;
+#else
+  typedef LEX_STRING mrn_foreign_key_name;
+#endif
+
+#if MYSQL_VERSION_ID >= 100302 && defined(MRN_MARIADB_P)
+#  define MRN_KEY_NAME_IS_LEX_STRING
+#  define MRN_FIELD_FIELD_NAME_IS_LEX_STRING
+#  define MRN_FIELD_SET_USE_LEX_STRING
+#  define MRN_CREATE_FIELD_USE_LEX_STRING
+#endif
+
+#ifdef MRN_KEY_NAME_IS_LEX_STRING
+#  define KEY_NAME(key) (key)->name.str, (key)->name.length
+#  define KEY_NAME_EQUAL_KEY(key1, key2)                \
+  ((key1)->name.length == (key2)->name.length &&        \
+   strncmp((key1)->name.str,                            \
+           (key2)->name.str,                            \
+           (key1)->name.length) == 0)
+#  define KEY_NAME_FORMAT "%.*s"
+#  define KEY_NAME_FORMAT_VALUE(key)            \
+  static_cast<int>((key)->name.length),         \
+  (key)->name.str
+#else
+#  define KEY_NAME(key) (key)->name, strlen((key)->name)
+#  define KEY_NAME_EQUAL_KEY(key1, key2)        \
+  strcmp((key1)->name, (key2)->name) == 0
+#  define KEY_NAME_FORMAT "%s"
+#  define KEY_NAME_FORMAT_VALUE(key) (key)->name
+#endif
+
+#ifdef MRN_FIELD_FIELD_NAME_IS_LEX_STRING
+#  define FIELD_NAME(field)                             \
+  (field)->field_name.str, (field)->field_name.length
+#  define FIELD_NAME_EQUAL(field, name)                                 \
+  ((field)->field_name.length == strlen(name) &&                        \
+   strncmp((field)->field_name.str, name, (field)->field_name.length) == 0)
+#  define FIELD_NAME_EQUAL_STRING(field, string)                        \
+  ((field)->field_name.length == string->length &&                      \
+   strncmp((field)->field_name.str, string->str, string->length) == 0)
+#  define FIELD_NAME_EQUAL_FIELD(field1, field2)                        \
+  ((field1)->field_name.length == (field2)->field_name.length &&        \
+   strncmp((field1)->field_name.str,                                    \
+           (field2)->field_name.str,                                    \
+           (field1)->field_name.length) == 0)
+#  define FIELD_NAME_TO_LEX_STRING(thd, field)                          \
+  thd_make_lex_string((thd),                                            \
+                      NULL,                                             \
+                      (field)->field_name.str,                          \
+                      (field)->field_name.length,                       \
+                      TRUE);
+#  define FIELD_NAME_FORMAT "%.*s"
+#  define FIELD_NAME_FORMAT_VALUE(field)                                \
+  static_cast<int>((field)->field_name.length), (field)->field_name.str
+#else
+#  define FIELD_NAME(field)                             \
+  (field)->field_name, strlen((field)->field_name)
+#  define FIELD_NAME_EQUAL(field, name)                                 \
+  (strcmp((field)->field_name, name) == 0)
+#  define FIELD_NAME_EQUAL_STRING(field, string)                        \
+  (strlen((field)->field_name) == string->length &&                     \
+   strncmp((field)->field_name, string->str, string->length) == 0)
+#  define FIELD_NAME_EQUAL_FIELD(field1, field2)                        \
+  (strcmp((field1)->field_name, (field2)->field_name) == 0)
+#  define FIELD_NAME_TO_LEX_STRING(thd, field)          \
+   thd_make_lex_string((thd),                           \
+                       NULL,                            \
+                       (field)->field_name,             \
+                       strlen((field)->field_name),     \
+                       TRUE);
+#  define FIELD_NAME_FORMAT "%s"
+#  define FIELD_NAME_FORMAT_VALUE(field) (field)->field_name
+#endif
+
+#if MYSQL_VERSION_ID >= 100302 && defined(MRN_MARIADB_P)
+#  define MRN_KEY_PART_SPEC_FIELD_NAME_USE_CONST_STRING
+#endif
+
+#ifdef MRN_KEY_PART_SPEC_FIELD_NAME_USE_CONST_STRING
+  typedef LEX_CSTRING mrn_key_part_spec_field_name;
+#else
+  typedef LEX_STRING mrn_key_part_spec_field_name;
+#endif
+
+#if MYSQL_VERSION_ID >= 100302 && defined(MRN_MARIADB_P)
+#  define MRN_THD_MAKE_LEX_STRING_USE_CONST_STRING
+#endif
+
+#ifdef MRN_THD_MAKE_LEX_STRING_USE_CONST_STRING
+  typedef MYSQL_CONST_LEX_STRING mrn_thd_lex_string;
+#else
+  typedef MYSQL_LEX_STRING mrn_thd_lex_string;
+#endif
+
 #if defined(MRN_MARIADB_P) && MYSQL_VERSION_ID >= 100000
 #  define mrn_init_alloc_root(PTR, SZ1, SZ2, FLAG) \
   init_alloc_root(PTR, SZ1, SZ2, FLAG)
@@ -101,6 +205,10 @@
 #  define MRN_ITEM_HAVE_ITEM_NAME
 #endif
 
+#if MYSQL_VERSION_ID >= 100302 && defined(MRN_MARIADB_P)
+#  define MRN_ITEM_ITEM_NAME_IS_LEX_STRING
+#endif
+
 #if MYSQL_VERSION_ID < 100000
 #  define MRN_HAVE_TABLE_DEF_CACHE
 #endif
@@ -362,6 +470,8 @@
 
 #if defined(MRN_MARIADB_P) &&                                           \
   ((MYSQL_VERSION_ID >= 100207) ||                                      \
+   ((MYSQL_VERSION_ID >= 100126) && (MYSQL_VERSION_ID < 100200)) ||     \
+   ((MYSQL_VERSION_ID >= 100032) && (MYSQL_VERSION_ID < 100100)) ||     \
    ((MYSQL_VERSION_ID >= 50557) && (MYSQL_VERSION_ID < 100000)))
 #  define mrn_create_partition_name(out,                                \
                                     out_length,                         \
@@ -416,4 +526,63 @@
 #define MRN_FREE_VARIABLE_LENGTH_ARRAYS(variable_name) \
   my_free(variable_name)
 
-#endif /* MRN_MYSQL_COMPAT_H_ */
+#if ((defined(MRN_MARIADB_P) && MYSQL_VERSION_ID >= 100203)) || \
+  (!defined(MRN_MARIADB_P) && MYSQL_VERSION_ID >= 50711)
+#  define MRN_ALTER_INPLACE_INFO_ADD_VIRTUAL_COLUMN \
+  Alter_inplace_info::ADD_VIRTUAL_COLUMN
+#  define MRN_ALTER_INPLACE_INFO_ADD_STORED_BASE_COLUMN \
+  Alter_inplace_info::ADD_STORED_BASE_COLUMN
+#  define MRN_ALTER_INPLACE_INFO_ADD_STORED_GENERATED_COLUMN \
+  Alter_inplace_info::ADD_STORED_GENERATED_COLUMN
+#else
+#  define MRN_ALTER_INPLACE_INFO_ADD_VIRTUAL_COLUMN 0
+#  define MRN_ALTER_INPLACE_INFO_ADD_STORED_BASE_COLUMN \
+  Alter_inplace_info::ADD_COLUMN
+#  define MRN_ALTER_INPLACE_INFO_ADD_STORED_GENERATED_COLUMN 0
+#endif
+
+#if (defined(HA_CAN_VIRTUAL_COLUMNS) || defined(HA_GENERATED_COLUMNS))
+#  define MRN_SUPPORT_GENERATED_COLUMNS
+#endif
+
+#ifdef MRN_MARIADB_P
+#  if (MYSQL_VERSION_ID >= 100200)
+#    define MRN_GENERATED_COLUMNS_FIELD_IS_VIRTUAL(field) \
+       (!field->stored_in_db())
+#    define MRN_GENERATED_COLUMNS_FIELD_IS_STORED(field) \
+       (field->vcol_info && field->vcol_info->is_stored())
+#  elif (MYSQL_VERSION_ID >= 50500)
+#    define MRN_GENERATED_COLUMNS_FIELD_IS_VIRTUAL(field) \
+       (!field->stored_in_db)
+#    define MRN_GENERATED_COLUMNS_FIELD_IS_STORED(field) \
+       (field->vcol_info && field->vcol_info->is_stored())
+#  else
+#    define MRN_GENERATED_COLUMNS_FIELD_IS_VIRTUAL(field) false
+#    define MRN_GENERATED_COLUMNS_FIELD_IS_STORED(field) false
+#  endif
+#else
+#  if (MYSQL_VERSION_ID >= 50708)
+#    define MRN_GENERATED_COLUMNS_FIELD_IS_VIRTUAL(field) \
+       (field->is_virtual_gcol())
+#    define MRN_GENERATED_COLUMNS_FIELD_IS_STORED(field) \
+       (field->is_gcol() && !field->is_virtual_gcol())
+#  elif (MYSQL_VERSION_ID >= 50706)
+#    define MRN_GENERATED_COLUMNS_FIELD_IS_VIRTUAL(field) \
+       (!field->stored_in_db)
+#    define MRN_GENERATED_COLUMNS_FIELD_IS_STORED(field) \
+       (field->gcol_info && field->gcol_info->get_field_stored())
+#  else
+#    define MRN_GENERATED_COLUMNS_FIELD_IS_VIRTUAL(field) false
+#    define MRN_GENERATED_COLUMNS_FIELD_IS_STORED(field) false
+#  endif
+#endif
+
+#ifdef MRN_MARIADB_P
+#  if (MYSQL_VERSION_ID >= 100203)
+#    define MRN_GENERATED_COLUMNS_UPDATE_VIRTUAL_FIELD(table, field) \
+       (table->update_virtual_field(field))
+#  else
+#    define MRN_GENERATED_COLUMNS_UPDATE_VIRTUAL_FIELD(table, field) \
+       (field->vcol_info->expr_item->save_in_field(field, 0))
+#  endif
+#endif
